`timescale 1ns / 1ps
//
// Copyright (C) 2005-2006 by egnite Software GmbH. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. Neither the name of the copyright holders nor the names of
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
// ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
// SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// For additional information see http://www.ethernut.de/
//

//
// $Log$
//

module MMC(PLCLK1, NWE, NOE, A, P26, D, MCMD, MDAT0, MCLK, NRESET, reg_MMC, mmc_irq);

  input NRESET;       // Board reset, low active.
  input PLCLK1;       // Clock B from programmable clock generator.

  // CPU interface.
  input [15:0]A;      // Address bus.
  inout [7:0]D;       // Data bus.
  input P26;          // Chip select 2, low active.
  input NWE;          // Write strobe, low active.
  input NOE;          // Read strobe, low active.

  // SPI interface.
  input MDAT0;        // SPI MISO line.
  output MCMD;        // SPI MOSI line.
  output MCLK;        // SPI SCK line.

  output [7:0]reg_MMC;
  output mmc_irq;


  reg [7:0]reg_MMC;
  reg [3:0]MMC_COUNT;
  reg MCMD;
  reg MMC_CLK_DIS;
  reg MCLK;
  reg mmc_irq;

  assign MMC_NWE = F_MMC_NWE(A,P26,NWE,NRESET) ;

  function F_MMC_NWE;
    input [15:0]A;
    input P26;
    input NWE;
    input NRESET;

    if(NRESET == 0)
      F_MMC_NWE = 0;
    else if((A == 8'h14) & (P26 == 0) & (NWE == 0))
      F_MMC_NWE = 0;
    else
      F_MMC_NWE = 1;
  endfunction 

  always @(posedge MCLK or negedge MMC_NWE) // MMC Lesen
  begin
    if(MMC_NWE == 0)
    begin
      if(NRESET == 0) // NRESET MMC Register wird auf 0 gesetzt MMC_COUNT wird auf 8 gesetzt damit der MMC_CLK gestopt wird
      begin
        reg_MMC <= 8'b0;
        MMC_COUNT = 9;
        mmc_irq = 0;
      end
      else    // NWE Daten werden in's reg_MMC geschrieben MMC_COUNT wird auf 0 gesetzt  MMC_CLK wird aktiviert
      begin
        reg_MMC <= D;
        MMC_COUNT = 0;
        mmc_irq = 0;
      end
    end
    else if(MCLK == 1)	// Daten werden von der MMC in reg_MMC geschrieben
    begin
      if(MMC_COUNT <  8)
      begin
        reg_MMC <= {reg_MMC[6:0],MDAT0};
        MMC_COUNT = MMC_COUNT+1;
        if(MMC_COUNT == 7)
          mmc_irq = 1;
      end
    end
  end


  always @(negedge PLCLK1)	// MMC Clock
  begin
    if((MMC_COUNT <  8) & (MMC_NWE == 1)) begin
      MCLK = MCLK + 1;
    end else
      MCLK = 1;
  end

  always @(negedge MCLK)	// MMC Schreiben
  begin
    if(MMC_COUNT <  8)
    begin
      MCMD = reg_MMC[7];	
    end
  end

endmodule
