`timescale 1ns / 1ps
//
// Copyright (C) 2005-2006 by egnite Software GmbH. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. Neither the name of the copyright holders nor the names of
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
// ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
// SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// For additional information see http://www.ethernut.de/
//

//
// $Log$
//

module RS232(CFON, CNFOFF, CRD, CDTR, CRTS, CTD, P14, P21, P15, P22, CDSR, reg0_RSCR);

  // CPU interface.
  input P14;          // UART0 transmit data.
  output P15;         // UART0 receive data.
  input P21;          // UART1 transmit data.
  output P22;         // UART1 receive data.

  // RS232 interface.
  output CTD;         // Transmit data.
  input CRD;          // Receive data.
  output CRTS;        // Ready to send.
  output CDTR;        // Data terminal ready.
  input CDSR;         // Data set ready.
  output CFON;        // Force on, high active.
  output CNFOFF;      // Force off, low active.

  // RS232 command register (NPL_RSCR).
  input [7:0]reg0_RSCR;

  wire CDTR_buf;

  // Connect the register to the force on/off outputs. 
  assign CNFOFF = ~reg0_RSCR[1];
  assign CFON	= reg0_RSCR[0];
  //assign CNFOFF = 1;
  //assign CFON	= 1;

  // Connect the register to the RTS output.
  assign CRTS	= ~reg0_RSCR[3];

  assign CDTR = ((reg0_RSCR[7] & reg0_RSCR[5]) | (~reg0_RSCR[7] & reg0_RSCR[6])) ? CDTR_buf : ~reg0_RSCR[2];

  // Route primary receive data to UART0.
  assign P15 = (~reg0_RSCR[7] & reg0_RSCR[5]) ? CRD : 1'bz;
  // Route secondary receive data to UART0.
  assign P15 = ( reg0_RSCR[7] & reg0_RSCR[5]) ? CDSR : 1'bz;

  // Route primary transmit data from UART0.
  assign CTD = (~reg0_RSCR[7] & reg0_RSCR[5]) ? P14 : 1'bz;
  // 
  assign CDTR_buf = ( reg0_RSCR[7] & reg0_RSCR[5]) ? P14 : P21;

  // Route primary receive data to UART1.
  assign P22 = ( reg0_RSCR[7] & reg0_RSCR[6]) ? CRD : 1'bz;
  // Route secondary receive data to UART1.
  assign P22 = (~reg0_RSCR[7] & reg0_RSCR[6]) ? CDSR : 1'bz;

  // Route primary transmit data from UART1.
  assign CTD = ( reg0_RSCR[7] & reg0_RSCR[6]) ? P21 : 1'bz;
  // 
  //assign CDTR_buf = (~reg0_RSCR[7] & reg0_RSCR[6]) ? P21 : 1'bz;

endmodule
