`timescale 1ns / 1ps
//
// Copyright (C) 2005-2006 by egnite Software GmbH. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. Neither the name of the copyright holders nor the names of
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
// ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
// SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// For additional information see http://www.ethernut.de/
//

//
// $Log$
//

//
// Main module for Ethernut 3.0 CPLD.
//
module ethernut30d(A, NWE, NOE, NRESET, P26, ULED, D, P14, P15, CFON, CNFOFF,
CRD, CDTR, CRTS, CTD, NRESET, P14, P21, P15, P22, CDSR, CNINVAL, P9, MCD, 
CCTS, CDCD, CRI, ALARM, WAKEUP, FBUSY, CAR, CDR, MCMD, MDAT0, MCLK, CS4,
PLCLK1, NCRD, NCWR, MDAT3, PNCS);

  input NRESET;       // Board reset, low active.
  input PLCLK1;       // Clock B from programmable clock generator.
  input ALARM;        // RTC alarm.
  input WAKEUP;       // LAN wakeup.
  input FBUSY;        // Flash busy.
  output ULED;        // User LED.

  // CPU interface.
  input [15:0]A;      // Address bus.
  inout [7:0]D;       // Data bus.
  input P26;          // Chip select 2, low active.
  input CS4;          // Chip select 4, high active.
  input NWE;          // Write strobe, low active.
  input NOE;          // Read strobe, low active.
  output P9;          // Interrupt request 0.
  input P14;          // UART0 transmit data.
  output P15;         // UART0 receive data.
  input P21;          // UART1 transmit data.
  output P22;         // UART1 receive data.

  // RS232 interface.
  output CTD;         // Transmit data.
  input CRD;          // Receive data.
  output CRTS;        // Ready to send.
  input CCTS;         // Clear to send.
  output CDTR;        // Data terminal ready.
  input CDSR;         // Data set ready.
  input CDCD;         // Data carrier detect.
  input CRI;          // Ring indicator.
  input CNINVAL;      // Invalid signal levels, low active.
  output CFON;        // Force on, high active.
  output CNFOFF;      // Force off, low active.

  // MMC interface.
  input MDAT0;        // SPI MISO line.
  output MCMD;        // SPI MOSI line
  output MCLK;        // SPI SCK line.
  output MDAT3;       // Card select line.
  input MCD;          // Card detect sense.

  // Expansion interface.
  output [15:0]CAR;   // Expansion address bus.
  inout [7:0]CDR;     // Expansion data bus.
  output NCRD;        // Expansion read strobe.
  output NCWR;        // Expansion write strobe.
  output PNCS;        // Panel chip select.


  wire [7:0]reg0_RSCR;
  wire [7:0]reg0_IMR;
  wire [3:0]reg1_IMR;
  wire [7:0]reg0_MMC;
  wire [7:0]reg_MMC;
  wire mmc_irq;
  wire [2:0]reg1_MMC;
  wire [6:0]RESET0_irq;
  wire [3:0]RESET1_irq;
  wire NWCS2;
  wire NOCS2;

  reg [7:0]reg0_irq;  // Signal latch register, low byte.
  reg [3:0]reg1_irq;  // Signal latch register, high byte.

  parameter AD0_ICR = 8'h10;
  parameter AD1_ICR = 8'h11;

  assign RESET1_irq = clear1_irq(A,D,P26,NWE,NRESET);

  //
  // Clear interrupt.
  //
  function [3:0]clear1_irq;
  input [15:0]A;
    input [3:0]D;
    input P26;
    input NWE;
    input NRESET;
    if(NRESET == 0)
      clear1_irq = 4'b0000;
    else if((A == AD1_ICR) & (P26 == 0) & (NWE == 1))
      clear1_irq = ~D;
    else
      clear1_irq = 4'b1111;
  endfunction

  // Update NPL_RSINVAL bit.
  always @(posedge RESET1_irq[0] or posedge CNINVAL)
  begin
    if(CNINVAL == 1)
      reg1_irq[0] = 1;
    else
      reg1_irq[0] = 0;
  end

  // Update NPL_NRSINVAL bit.
  always 
    @(posedge RESET1_irq[1] or negedge CNINVAL)
    begin
      if(CNINVAL == 0)
        reg1_irq[1] = 1;
      else
        reg1_irq[1] = 0;
    end

  // Update NPL_MMCD bit.
  always @(posedge RESET1_irq[2] or negedge MCD)
  begin
    if(MCD == 0)
      reg1_irq[2] = 1;
    else
      reg1_irq[2] = 0;
  end

  // Update NPL_NMMCD bit.
  always @(posedge RESET1_irq[3] or posedge MCD)
  begin
    if(MCD == 1)
      reg1_irq[3] = 1;
    else
      reg1_irq[3] = 0;
  end

  assign RESET0_irq = clear0_irq(A,D,P26,NWE,NRESET);

  function [6:0]clear0_irq ;
    input [15:0]A;
    input [6:0]D;
    input P26;
    input NWE;
    input NRESET;

    if(NRESET == 0)
      clear0_irq = 7'b0000000;
    else if((A == AD0_ICR)&(P26 == 0)&(NWE == 1))
      clear0_irq = ~D;
    else
      clear0_irq = 7'b1111111;
  endfunction

  // Update NPL_RSCTS bit.
  always @(posedge RESET0_irq[0] or negedge CCTS)
  begin
    if(CCTS == 0)
      reg0_irq[0] = 1;
    else
      reg0_irq[0] = 0;
  end

  always @(posedge RESET0_irq[1] or negedge CDSR)
  begin
    if(CDSR == 0)
      reg0_irq[1] = 1;
    else
      reg0_irq[1] = 0;
  end

  always @(posedge RESET0_irq[2] or negedge CDCD)
  begin
    if(CDCD == 0)
      reg0_irq[2] = 1;
    else
      reg0_irq[2] = 0;
  end

  always @(posedge RESET0_irq[3] or posedge CRI)
  begin
    if(CRI == 1)
      reg0_irq[3] = 1;
    else
      reg0_irq[3] = 0;
  end

  always @(posedge RESET0_irq[4] or negedge ALARM)
  begin
    if(ALARM == 0)
      reg0_irq[4] = 1;
    else
      reg0_irq[4] = 0;
  end

  always @(posedge RESET0_irq[5] or posedge WAKEUP)
  begin
    if(WAKEUP == 1)
      reg0_irq[5] = 1;
    else
      reg0_irq[5] = 0;
  end

  always @(posedge RESET0_irq[6] or negedge FBUSY)
  begin
    if(FBUSY == 0)
      reg0_irq[6] = 1;
    else
      reg0_irq[6] = 0;
  end

  // Set CPU IRQ
  assign P9  = ~((|(reg0_irq & reg0_IMR)) | (|(reg1_irq & reg1_IMR)));

  assign PNCS = reg1_MMC[1];
  assign ULED = reg1_MMC[2];

  // Expansion port address/data bus
  assign NWCS2 = ~NWE & CS4;
  assign NOCS2 = ~NOE & CS4;
  assign CAR = CS4 ? A : 16'bzzzzzzzzzzzzzzzz;
  assign NCRD = CS4 ? NOE : 1'bz;
  assign NCWR = CS4 ? NWE : 1'bz;
  assign CDR = NWCS2 ? D : 8'bzzzzzzzz;
  assign D   = NOCS2 ? CDR : 8'bzzzzzzzz;

  // MMC chip select.
  assign MDAT3 = reg1_MMC[0];

  REGISTER REG(NOE, NWE, NRESET, A, P26, D, reg0_RSCR, reg0_IMR, reg1_IMR, 
  reg0_irq, reg1_irq, reg_MMC, reg1_MMC, mmc_irq);

  // RS232 interface processing.
  RS232 rs(CFON, CNFOFF, CRD, CDTR, CRTS, CTD, P14, P21, P15, P22, CDSR, reg0_RSCR);

  // MMC interface processing.
  MMC mmc(PLCLK1, NWE, NOE, A, P26, D, MCMD, MDAT0, MCLK, NRESET, reg_MMC, mmc_irq);

endmodule
